/// \file spiuart.c VS1000 UART Audio Module controlling software
/// \author Panu-Kristian Poiksalo, VLSI Solution Oy 2008


#define SPI_CLOCK_DIVIDER 2


#include <stdio.h> //Standard io
#include <ctype.h>
//#include <stdlib.h> // VS_DSP Standard Library
#include <vs1000.h> // VS1000B register definitions
//#include <vectors.h> // VS1000B vectors (interrupts and services)
#include <fat.h> // Read Only Fat Filesystem
#include <string.h> // memcpy etc
#include <player.h> // VS1000B default ROM player
#include <audio.h> // DAC output
//#include <codec.h> // CODEC interface
#include <mapper.h>

#include "uartRxInt.h"
#include "spiflash.h"
#include "sbfs.h"
#inclu


// Do we want to get debug screen output? It's available if the code
// is loaded with vs3emu (build script) with RS-232 cable.
#if PRINT_VS3EMU_DEBUG_MESSAGES
#define do__not__puts(x) Writeln(x)
#define do__not__puthex(x) puthex(x)
#else
#define do__not__puts(a)
#define do__not__puthex(a)
#define PrintCache()
#endif

u_int32 bytesReceived;

const u_int16 __x charToNum[] = {0,1,2,3,4,5,6,7,8,9,0,0,0,0,0,0,0,10,11,12,13,14,15};
const u_int16 __y charToNumH[] = {0*16,1*16,2*16,3*16,4*16,5*16,6*16,7*16,8*16,9*16,0*16,0*16,0*16,0*16,0*16,0*16,0*16,10*16,11*16,12*16,13*16,14*16,15*16};

auto u_int16 ChCh2Hex(register char msb, register char lsb){
	bytesReceived++;
	return ((charToNumH-'0')[msb]) | (charToNum-'0')[lsb];
}

void BusyWait(u_int16 i){
	while(i--) BusyWait10();
}

char *cp;
char filename[112];	

#define buf minifatBuffer


extern struct FsMapper *map;

auto void GetFileName(){
	char *cp = filename;
	char ch;
	//Receive file name until '<'
	Writeln("Name:");	
	while (((ch=MyGetChar()) >= ' ') && (ch != '<')){
		putch2(ch);
		*cp++ = ch;
	}
	*cp=0; //Ending zero of pchar filename
	putch2('\r');
}


void main(void) {
	u_int16 i,bl;
	
	
	InitAudio();
 	SetRate(8000);
	//Mic input GPIOs
	// Enable Interrupts
	PERIP(INT_ENABLEH) = 0;
	PERIP(INT_ENABLEL) = INTF_RX | INTF_DAC;
	
	// Drop from USB
	PERIP(SCI_STATUS) &= ~SCISTF_USB_PULLUP_ENA;
	//PERIP(USB_CONFIG) = 0x8000U;
  	
	PERIP(GPIO1_ODATA) |=  LED1|LED2;
	PERIP(GPIO1_DDR)   |=  LED1|LED2;
	PERIP(GPIO1_MODE)  &= ~(LED1|LED2);
  	
	player.volumeOffset = 0;
	player.pauseOn = 0;  	
	keyOld = KEY_POWER;
	keyOldTime = -32767;
	
	SetHookFunction((u_int16)OpenFile, SbfsOpenFile);
	SetHookFunction((u_int16)IdleHook, NullHook); //Disable keyboard scanning
	SetHookFunction((u_int16)ReadDiskSector, EeReadDiskSector);

 	InitSpi(SPI_CLOCK_DIVIDER);  

	goto print_ident;

	SetSerialInterruptVector();	
	while(1) {		
		char opcode,ch;
		Writeln("\r#");
		opcode = toupper(MyGetChar());
		putch2(opcode);
		putch2('\r');
		switch(opcode) {
		case 'F': //Files
			SbfsOpenFile(FILE_LIST,NULL);
			break;
		case 'Q': //Quick
			waitStates = 0;
			Writeln("Quick");
			break;
		case 'S': //Slow
			waitStates = 1;
			Writeln("Slow");
			break;
		case 'E': //Erase all
			for (bl=FIRST_BLOCK; bl<LAST_BLOCK; bl+=ERASEBLOCK_SIZE){
				EeErase4K(bl);
			}			
			break;
		case 'D': //Delete file
			GetFileName();
			SbfsEraseFile(filename);		
			break;
		case 'P': //Play file
			GetFileName();
			if (SbfsOpenFile(0,filename)){
				Writeln("Play..");
				PlayCurrentFile();
			}else{
				Writeln("File not found.");			
			}
			
			break;
		case 'L': //Download file
		case 'R': //Read file
			GetFileName();
			if (SbfsOpenFile(0,filename)){
				char c;
				while (c=0,ReadFile(&c,1,1)) {
					if (opcode=='R') {
						putch2(c);
					} else {
						putch2(hex[c>>4]);
						putch2(hex[c&15]);
					}
				}				
			} else {
				Writeln("File not found.");
			}

			break;
		case 'U': //Upload file
			GetFileName();
			if (!SbfsCreateFile(filename)) {
				cp = buf;
				Writeln("Send upcase hex now and end with \".\"");
				BusyWait10();
				putch2('>');
				BusyWait10();
				bytesReceived = 0;
				while(1) {
					u_int16 b;
					if ((ch = MyGetChar()) == '.') goto out;
					while (ch<'0') {
						ch = MyGetChar();
					}
					b=ChCh2Hex(ch,MyGetChar());
					if ((bytesReceived & 0x3ff) == 1) {
						putch2('~');
					}
					SbfsWriteByte(b);
				}
				out:
				SbfsCloseFile(filename);
				putch2('\r');
				puthex32(bytesReceived);
				Writeln("bytes received.\r");
			}
			break;
		case 'H': //Help
			print_ident:
			Writeln("VS1000 UART Audio Module V1.01.\rCommands: H)elp F)ilelist U)pload downL)oad R)ead E)raseAll D)eleteFile P)lay\rQ)uick S)low\r");
			break;
		case '\r':
			// Intentional fall-through
		case '\n':
			break;
		default:
			Writeln("Unknown command");
			break;
		}
	}	
}

